@@ -13,7 +13,6 @@
  *	David Dawes, Andrew E. Mileski, Leonard N. Zubkoff,
  *	Guy DESBIEF, Itai Nahshon.
  */
-/* $XFree86: xc/programs/Xserver/hw/xfree86/drivers/cirrus/lg_driver.c,v 1.49 2003/11/03 05:11:09 tsi Exp $ */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -29,9 +28,6 @@
 
 #include "compiler.h"
 
-/* Drivers for PCI hardware need this */
-#include "xf86PciInfo.h"
-
 /* Drivers that need to access the PCI config space directly need this */
 #include "xf86Pci.h"
 
@@ -79,20 +75,20 @@
 
 /* Mandatory functions */
 Bool LgPreInit(ScrnInfoPtr pScrn, int flags);
-Bool LgScreenInit(int Index, ScreenPtr pScreen, int argc, char **argv);
-Bool LgEnterVT(int scrnIndex, int flags);
-void LgLeaveVT(int scrnIndex, int flags);
-static Bool	LgCloseScreen(int scrnIndex, ScreenPtr pScreen);
+Bool LgScreenInit(SCREEN_INIT_ARGS_DECL);
+Bool LgEnterVT(VT_FUNC_ARGS_DECL);
+void LgLeaveVT(VT_FUNC_ARGS_DECL);
+static Bool	LgCloseScreen(CLOSE_SCREEN_ARGS_DECL);
 static Bool	LgSaveScreen(ScreenPtr pScreen, Bool mode);
 
 /* Required if the driver supports mode switching */
-Bool LgSwitchMode(int scrnIndex, DisplayModePtr mode, int flags);
+Bool LgSwitchMode(SWITCH_MODE_ARGS_DECL);
 /* Required if the driver supports moving the viewport */
-void LgAdjustFrame(int scrnIndex, int x, int y, int flags);
+void LgAdjustFrame(ADJUST_FRAME_ARGS_DECL);
 
 /* Optional functions */
-void LgFreeScreen(int scrnIndex, int flags);
-ModeStatus LgValidMode(int scrnIndex, DisplayModePtr mode,
+void LgFreeScreen(FREE_SCREEN_ARGS_DECL);
+ModeStatus LgValidMode(SCRN_ARG_TYPE arg, DisplayModePtr mode,
 		       Bool verbose, int flags);
 
 /* Internally used functions */
@@ -259,7 +255,7 @@ LgFreeRec(ScrnInfoPtr pScrn)
 {
 	if (pScrn->driverPrivate == NULL)
 		return;
-	xfree(pScrn->driverPrivate);
+	free(pScrn->driverPrivate);
 	pScrn->driverPrivate = NULL;
 }
 
@@ -365,6 +361,7 @@ LgPreInit(ScrnInfoPtr pScrn, int flags)
 		return FALSE;
 
 	hwp = VGAHWPTR(pScrn);
+	vgaHWSetStdFuncs(hwp);
 	vgaHWGetIOBase(hwp);
 
 	/* Allocate the LgRec driverPrivate */
@@ -373,7 +370,12 @@ LgPreInit(ScrnInfoPtr pScrn, int flags)
 
 	pCir = CIRPTR(pScrn);
 	pCir->pScrn = pScrn;
+
+#if GET_ABI_MAJOR(ABI_VIDEODRV_VERSION) < 12
 	pCir->PIOReg = hwp->PIOOffset + 0x3CE;
+#else
+	pCir->PIOReg = 0x3CE;
+#endif
 
 	/* Get the entity, and make sure it is PCI. */
 	pCir->pEnt = xf86GetEntityInfo(pScrn->entityList[0]);
@@ -454,7 +456,7 @@ LgPreInit(ScrnInfoPtr pScrn, int flags)
 	xf86CollectOptions(pScrn, NULL);
 
 	/* Process the options */
-	if (!(pCir->Options = xalloc(sizeof(LgOptions))))
+	if (!(pCir->Options = malloc(sizeof(LgOptions))))
 		return FALSE;
 	memcpy(pCir->Options, LgOptions, sizeof(LgOptions));
 	xf86ProcessOptions(pScrn->scrnIndex, pScrn->options, pCir->Options);
@@ -795,9 +797,16 @@ LgPreInit(ScrnInfoPtr pScrn, int flags)
 
 	/* Load XAA if needed */
 	if (!pCir->NoAccel) {
-		if (!xf86LoadSubModule(pScrn, "xaa")) {
-			LgFreeRec(pScrn);
-			return FALSE;
+#ifdef HAVE_XAA_H
+		if (!xf86LoadSubModule(pScrn, "xaa"))
+#else
+		if (1)
+#endif
+                {
+			xf86DrvMsg(pScrn->scrnIndex, X_INFO,
+				   "Falling back to shadowfb\n");
+			pCir->NoAccel = TRUE;
+			pCir->shadowFB = TRUE;
 		}
 	}
 
@@ -1225,10 +1234,10 @@ LgRestore(ScrnInfoPtr pScrn)
 /* This gets called at the start of each server generation */
 
 Bool
-LgScreenInit(int scrnIndex, ScreenPtr pScreen, int argc, char **argv)
+LgScreenInit(SCREEN_INIT_ARGS_DECL)
 {
 	/* The vgaHW references will disappear one day */
-	ScrnInfoPtr pScrn;
+	ScrnInfoPtr pScrn = xf86ScreenToScrn(pScreen);
 	vgaHWPtr hwp;
 	CirPtr pCir;
 	int i, ret;
@@ -1240,11 +1249,6 @@ LgScreenInit(int scrnIndex, ScreenPtr pScreen, int argc, char **argv)
 	ErrorF("LgScreenInit\n");
 #endif
 
-	/*
-	 * First get the ScrnInfoRec
-	 */
-	pScrn = xf86Screens[pScreen->myNum];
-
 	hwp = VGAHWPTR(pScrn);
 
 	hwp->MapSize = 0x10000;		/* Standard 64k VGA window */
@@ -1274,7 +1278,7 @@ LgScreenInit(int scrnIndex, ScreenPtr pScreen, int argc, char **argv)
 	LgSaveScreen(pScreen, SCREEN_SAVER_ON);
 
 	/* Set the viewport */
-	LgAdjustFrame(scrnIndex, pScrn->frameX0, pScrn->frameY0, 0);
+	LgAdjustFrame(ADJUST_FRAME_ARGS(pScrn, pScrn->frameX0, pScrn->frameY0));
 
 	/*
 	 * The next step is to setup the screen's visuals, and initialise the
@@ -1314,7 +1318,7 @@ LgScreenInit(int scrnIndex, ScreenPtr pScreen, int argc, char **argv)
 	
 	if(pCir->shadowFB) {
 	    pCir->ShadowPitch = BitmapBytePad(pScrn->bitsPerPixel * width);
-	    pCir->ShadowPtr = xalloc(pCir->ShadowPitch * height);
+	    pCir->ShadowPtr = malloc(pCir->ShadowPitch * height);
 	    displayWidth = pCir->ShadowPitch / (pScrn->bitsPerPixel >> 3);
 	    FbBase = pCir->ShadowPtr;
 	} else {
@@ -1337,7 +1341,7 @@ LgScreenInit(int scrnIndex, ScreenPtr pScreen, int argc, char **argv)
 				displayWidth,pScrn->bitsPerPixel);
 	    break;
 	default:
-		xf86DrvMsg(scrnIndex, X_ERROR,
+		xf86DrvMsg(pScrn->scrnIndex, X_ERROR,
 			   "X11: Internal error: invalid bpp (%d) in LgScreenInit\n",
 			   pScrn->bitsPerPixel);
 		ret = FALSE;
@@ -1376,10 +1380,12 @@ LgScreenInit(int scrnIndex, ScreenPtr pScreen, int argc, char **argv)
 	 */
 	xf86SetBlackWhitePixels(pScreen);
 
+#ifdef HAVE_XAA_H
 	if (!pCir->NoAccel) { /* Initialize XAA functions */
 		if (!LgXAAInit(pScreen))
 			xf86DrvMsg(pScrn->scrnIndex, X_ERROR, "Could not initialize XAA\n");
 	}
+#endif
 #if 1
 	pCir->DGAModeInit = LgModeInit;
 	if (!CirDGAInit(pScreen))
@@ -1436,9 +1442,10 @@ LgScreenInit(int scrnIndex, ScreenPtr pScreen, int argc, char **argv)
 
 /* Usually mandatory */
 Bool
-LgSwitchMode(int scrnIndex, DisplayModePtr mode, int flags)
+LgSwitchMode(SWITCH_MODE_ARGS_DECL)
 {
-	return LgModeInit(xf86Screens[scrnIndex], mode);
+	SCRN_INFO_PTR(arg);
+	return LgModeInit(pScrn, mode);
 }
 
 #define ROUND_DOWN(x, mod)	(((x) / (mod)) * (mod))
@@ -1450,9 +1457,9 @@ LgSwitchMode(int scrnIndex, DisplayModePtr mode, int flags)
  */
 /* Usually mandatory */
 void
-LgAdjustFrame(int scrnIndex, int x, int y, int flags)
+LgAdjustFrame(ADJUST_FRAME_ARGS_DECL)
 {
-	ScrnInfoPtr pScrn = xf86Screens[scrnIndex];
+	SCRN_INFO_PTR(arg);
 	int Base, tmp;
 	CirPtr pCir = CIRPTR(pScrn);
 	vgaHWPtr hwp = VGAHWPTR(pScrn);
@@ -1538,9 +1545,9 @@ LgAdjustFrame(int scrnIndex, int x, int y, int flags)
 
 /* Mandatory */
 Bool
-LgEnterVT(int scrnIndex, int flags)
+LgEnterVT(VT_FUNC_ARGS_DECL)
 {
-	ScrnInfoPtr pScrn = xf86Screens[scrnIndex];
+	SCRN_INFO_PTR(arg);
 	CirPtr pCir = CIRPTR(pScrn);
 #ifdef LG_DEBUG
 	ErrorF("LgEnterVT\n");
@@ -1565,9 +1572,9 @@ LgEnterVT(int scrnIndex, int flags)
 
 /* Mandatory */
 void
-LgLeaveVT(int scrnIndex, int flags)
+LgLeaveVT(VT_FUNC_ARGS_DECL)
 {
-	ScrnInfoPtr pScrn = xf86Screens[scrnIndex];
+	SCRN_INFO_PTR(arg);
 	vgaHWPtr hwp = VGAHWPTR(pScrn);
 	CirPtr pCir = CIRPTR(pScrn);
 #ifdef LG_DEBUG
@@ -1593,9 +1600,9 @@ LgLeaveVT(int scrnIndex, int flags)
 
 /* Mandatory */
 static Bool
-LgCloseScreen(int scrnIndex, ScreenPtr pScreen)
+LgCloseScreen(CLOSE_SCREEN_ARGS_DECL)
 {
-	ScrnInfoPtr pScrn = xf86Screens[scrnIndex];
+	ScrnInfoPtr pScrn = xf86ScreenToScrn(pScreen);
 	vgaHWPtr hwp = VGAHWPTR(pScrn);
 	CirPtr pCir = CIRPTR(pScrn);
 
@@ -1609,22 +1616,24 @@ LgCloseScreen(int scrnIndex, ScreenPtr pScreen)
 	CirUnmapMem(pCir, pScrn->scrnIndex);
 	}
 
+#ifdef HAVE_XAA_H
 	if (pCir->AccelInfoRec)
 		XAADestroyInfoRec(pCir->AccelInfoRec);
 	pCir->AccelInfoRec = NULL;
+#endif
 
 	if (pCir->CursorInfoRec)
 		xf86DestroyCursorInfoRec(pCir->CursorInfoRec);
 	pCir->CursorInfoRec = NULL;
 	if (pCir->DGAModes)
-		xfree(pCir->DGAModes);
+		free(pCir->DGAModes);
 	pCir->DGAnumModes = 0;
 	pCir->DGAModes = NULL;
 
 	pScrn->vtSema = FALSE;
 
 	pScreen->CloseScreen = pCir->CloseScreen;
-	return (*pScreen->CloseScreen)(scrnIndex, pScreen);
+	return (*pScreen->CloseScreen)(CLOSE_SCREEN_ARGS);
 }
 
 
@@ -1632,8 +1641,9 @@ LgCloseScreen(int scrnIndex, ScreenPtr pScreen)
 
 /* Optional */
 void
-LgFreeScreen(int scrnIndex, int flags)
+LgFreeScreen(FREE_SCREEN_ARGS_DECL)
 {
+	SCRN_INFO_PTR(arg);
 #ifdef LG_DEBUG
 	ErrorF("LgFreeScreen\n");
 #endif
@@ -1642,8 +1652,8 @@ LgFreeScreen(int scrnIndex, int flags)
 	 * get called routinely at the end of a server generation.
 	 */
 	if (xf86LoaderCheckSymbol("vgaHWFreeHWRec"))
-		vgaHWFreeHWRec(xf86Screens[scrnIndex]);
-	LgFreeRec(xf86Screens[scrnIndex]);
+		vgaHWFreeHWRec(pScrn);
+	LgFreeRec(pScrn);
 }
 
 
@@ -1651,7 +1661,7 @@ LgFreeScreen(int scrnIndex, int flags)
 
 /* Optional */
 ModeStatus
-LgValidMode(int scrnIndex, DisplayModePtr mode, Bool verbose, int flags)
+LgValidMode(SCRN_ARG_TYPE arg, DisplayModePtr mode, Bool verbose, int flags)
 {
 	int lace;
 
@@ -1677,14 +1687,14 @@ LgValidMode(int scrnIndex, DisplayModePtr mode, Bool verbose, int flags)
 static Bool
 LgSaveScreen(ScreenPtr pScreen, int mode)
 {
-	CirPtr pCir = CIRPTR(xf86Screens[pScreen->myNum]);
+	CirPtr pCir = CIRPTR(xf86ScreenToScrn(pScreen));
 	ScrnInfoPtr pScrn = NULL;
 	Bool unblank;
 
 	unblank = xf86IsUnblank(mode);
 
 	if (pScreen != NULL)
-	    pScrn = xf86Screens[pScreen->myNum];
+	    pScrn = xf86ScreenToScrn(pScreen);
 
 	if (pScrn != NULL && pScrn->vtSema) {
 	    if (unblank)
